@@ -7,6 +7,7 @@ MYPY = False
 if MYPY:
     from cura.PrinterOutput.Models.PrintJobOutputModel import PrintJobOutputModel
 
+
 class ClusterUM3PrinterOutputController(PrinterOutputController):
     def __init__(self, output_device):
         super().__init__(output_device)
@@ -15,6 +16,5 @@ class ClusterUM3PrinterOutputController(PrinterOutputController):
         self.can_control_manually = False
         self.can_send_raw_gcode = False
 
-    def setJobState(self, job: "PrintJobOutputModel", state: str):
-        data = "{\"action\": \"%s\"}" % state
-        self._output_device.put("print_jobs/%s/action" % job.key, data, on_finished=None)
+    def setJobState(self, job: "PrintJobOutputModel", state: str) -> None:
+        self._output_device.setJobState(job.key, state)
